// Copyright 2019, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Authors: Patrick Brosi <brosi@informatik.uni-freiburg.de>

#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include "osmfixer/index/StatIdx.h"

using osmfixer::StatIdx;
using osmfixer::OsmAttrs;
using osmfixer::Station;

// _____________________________________________________________________________
void StatIdx::readFromFile(const std::string& path) {
  std::ifstream f(path);

  std::string line;

  // first, parse nodes
  while (std::getline(f, line)) {
    // empty line is separator between blocks
    if (line.size() == 0) break;

    std::stringstream rec(line);
    size_t osmid;
    double lat, lng;
    OsmAttrs attrs;

    rec >> osmid;
    rec >> lat;
    rec >> lng;

    addStation(osmid, lat, lng, attrs);
  }

  std::cout << util::geo::getWKT(_bbox) << std::endl;
}

// _____________________________________________________________________________
void StatIdx::addStation(size_t osmid, double lat, double lng,
                         const OsmAttrs& attrs) {
  std::cout << "Record: " << osmid << " " << lat << " " << lng << std::endl;
  auto point = util::geo::latLngToWebMerc<double>(lat, lng);

  _stations.emplace_back(Station{osmid, point, attrs});

  // extend bounding box
  _bbox = util::geo::extendBox(point, _bbox);
}
