// Copyright 2019, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Authors: Patrick Brosi <brosi@informatik.uni-freiburg.de>

#include "osmfixer/server/StatServer.h"
#include "util/Misc.h"
#include "util/String.h"

using osmfixer::StatServer;
using osmfixer::Params;

// _____________________________________________________________________________
util::http::Answer StatServer::handle(const util::http::Req& req,
                                      int con) const {
  UNUSED(con);
  util::http::Answer a;
  try {
    Params params;
    auto cmd = parseUrl(req.url, req.payload, &params);

    if (cmd == "/") {
      a = util::http::Answer("200 OK", "osmfixer");
    } else {
      a = util::http::Answer("404 Not Found", "dunno");
    }
  } catch (std::runtime_error e) {
    a = util::http::Answer("400 Bad Request", e.what());
  } catch (std::invalid_argument e) {
    a = util::http::Answer("400 Bad Request", e.what());
  } catch (...) {
    a = util::http::Answer("500 Internal Server Error",
                           "Internal Server Error.");
  }

  a.params["Access-Control-Allow-Origin"] = "*";
  a.params["Server"] = "osmfixer";

  return a;
}

// _____________________________________________________________________________
std::string StatServer::parseUrl(std::string u, std::string pl,
                                 std::map<std::string, std::string>* params) {
  auto parts = util::split(u, '?');

  if (parts.size() > 1) {
    auto kvs = util::split(parts[1], '&');
    for (auto kv : kvs) {
      auto kvp = util::split(kv, '=');
      if (kvp.size() == 1) kvp.push_back("");
      (*params)[util::urlDecode(kvp[0])] = kvp[1];
    }
  }

  // also parse post data
  auto kvs = util::split(pl, '&');
  for (auto kv : kvs) {
    auto kvp = util::split(kv, '=');
    if (kvp.size() == 1) kvp.push_back("");
    (*params)[util::urlDecode(kvp[0])] = kvp[1];
  }

  return util::urlDecode(parts.front());
}
